#include "utils.h"

char *strpbrk(const char *s1, const char *s2)
{
    const char *c = s2;
    if (!*s1)
    return (char *) NULL;

    while (*s1) {
        for (c = s2; *c; c++) {
            if (*s1 == *c)
                break;
        }
        if (*c)
        break;
        s1++;
    }

    if (*c == '\0')
        s1 = NULL;

    return (char *) s1;
}

char *strtok_r(char *s, const char *delim, char **holder)
{
    if (s)
        *holder = s;
    do {
        s = strsep(holder, delim);
    } while (s && !*s);

    return s;
}

char *strtok(char *s, const char *delim)
{
    static char *holder;
    return strtok_r(s, delim, &holder);
}

char *strsep(char **stringp, const char *delim)
{
    char *s = *stringp;
    char *e;

    if (!s)
        return NULL;

    e = strpbrk(s, delim);
    if (e)
        *e++ = '\0';

    *stringp = e;
    return s;
}

static inline int digitval(int ch)
{
    if (ch >= '0' && ch <= '9') {
        return ch - '0';
    } else if (ch >= 'A' && ch <= 'Z') {
        return ch - 'A' + 10;
    } else if (ch >= 'a' && ch <= 'z') {
        return ch - 'a' + 10;
    } else {
        return -1;
    }
}

uint64_t strntoumax(const char *nptr, char **endptr, int base, size_t n)
{
    int minus = 0;
    uint64_t v = 0;
    int d;

    while (n && isspace((unsigned char)*nptr)) {
        nptr++;
        n--;
    }

    /* Single optional + or - */
    if (n) {
        char c = *nptr;
        if (c == '-' || c == '+') {
            minus = (c == '-');
            nptr++;
            n--;
        }
    }

    if (base == 0) {
        if (n >= 2 && nptr[0] == '0' &&
            (nptr[1] == 'x' || nptr[1] == 'X')) {
            n -= 2;
            nptr += 2;
            base = 16;
        } else if (n >= 1 && nptr[0] == '0') {
            n--;
            nptr++;
            base = 8;
        } else {
            base = 10;
        }
    } else if (base == 16) {
        if (n >= 2 && nptr[0] == '0' &&
            (nptr[1] == 'x' || nptr[1] == 'X')) {
            n -= 2;
            nptr += 2;
        }
    }

    while (n && (d = digitval(*nptr)) >= 0 && d < base) {
        v = v * base + d;
        n--;
        nptr++;
    }

    if (endptr)
        *endptr = (char *)nptr;

    return minus ? -v : v;
}

long strtol(const char *nptr, char **endptr, int base)
{
    return (long) strntoumax(nptr, endptr, base, ~(size_t) 0);
}

int atoi(const char *nptr)
{
    return (int) strntoumax(nptr, (char **)NULL, 10, ~(size_t) 0);
}
